@@ -1,6 +1,7 @@
 #![no_std]
 #![cfg_attr(test, deny(warnings))]
 #![deny(missing_docs)]
+#![allow(clippy::derive_partial_eq_without_eq)]
 
 //! Wrappers for total order on Floats.  See the [`OrderedFloat`] and [`NotNan`] docs for details.
 
@@ -27,7 +28,9 @@ use core::str::FromStr;
 use num_traits::float::FloatCore as Float;
 #[cfg(feature = "std")]
 pub use num_traits::Float;
-use num_traits::{Bounded, FromPrimitive, Num, NumCast, One, Signed, ToPrimitive, Zero};
+use num_traits::{
+    AsPrimitive, Bounded, FloatConst, FromPrimitive, Num, NumCast, One, Signed, ToPrimitive, Zero,
+};
 
 // masks for the parts of the IEEE 754 float
 const SIGN_MASK: u64 = 0x8000000000000000u64;
@@ -170,6 +173,20 @@ impl<T: Float + fmt::Display> fmt::Display for OrderedFloat<T> {
     }
 }
 
+impl<T: Float + fmt::LowerExp> fmt::LowerExp for OrderedFloat<T> {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.0.fmt(f)
+    }
+}
+
+impl<T: Float + fmt::UpperExp> fmt::UpperExp for OrderedFloat<T> {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.0.fmt(f)
+    }
+}
+
 impl From<OrderedFloat<f32>> for f32 {
     #[inline]
     fn from(f: OrderedFloat<f32>) -> f32 {
@@ -253,18 +270,6 @@ macro_rules! impl_ordered_float_binop {
             }
         }
 
-        impl<'a, T> $imp for &'a OrderedFloat<T>
-        where
-            &'a T: $imp,
-        {
-            type Output = OrderedFloat<<&'a T as $imp>::Output>;
-
-            #[inline]
-            fn $method(self, other: Self) -> Self::Output {
-                OrderedFloat((self.0).$method(&other.0))
-            }
-        }
-
         impl<'a, T> $imp<OrderedFloat<T>> for &'a OrderedFloat<T>
         where
             &'a T: $imp<T>,
@@ -301,19 +306,6 @@ macro_rules! impl_ordered_float_binop {
             }
         }
 
-        #[doc(hidden)] // Added accidentally; remove in next major version
-        impl<'a, T> $imp<&'a Self> for &'a OrderedFloat<T>
-        where
-            &'a T: $imp,
-        {
-            type Output = OrderedFloat<<&'a T as $imp>::Output>;
-
-            #[inline]
-            fn $method(self, other: &'a Self) -> Self::Output {
-                OrderedFloat((self.0).$method(&other.0))
-            }
-        }
-
         impl<T: $assign_imp> $assign_imp<T> for OrderedFloat<T> {
             #[inline]
             fn $assign_method(&mut self, other: T) {
@@ -477,6 +469,87 @@ impl<T: NumCast> NumCast for OrderedFloat<T> {
     }
 }
 
+macro_rules! impl_as_primitive {
+    (@ (NotNan<$T: ty>) => $(#[$cfg:meta])* impl (NotNan<$U: ty>) ) => {
+        $(#[$cfg])*
+        impl AsPrimitive<NotNan<$U>> for NotNan<$T> {
+            #[inline] fn as_(self) -> NotNan<$U> {
+                // Safety: `NotNan` guarantees that the value is not NaN.
+                unsafe {NotNan::new_unchecked(self.0 as $U) }
+            }
+        }
+    };
+    (@ ($T: ty) => $(#[$cfg:meta])* impl (NotNan<$U: ty>) ) => {
+        $(#[$cfg])*
+        impl AsPrimitive<NotNan<$U>> for $T {
+            #[inline] fn as_(self) -> NotNan<$U> { NotNan(self as $U) }
+        }
+    };
+    (@ (NotNan<$T: ty>) => $(#[$cfg:meta])* impl ($U: ty) ) => {
+        $(#[$cfg])*
+        impl AsPrimitive<$U> for NotNan<$T> {
+            #[inline] fn as_(self) -> $U { self.0 as $U }
+        }
+    };
+    (@ (OrderedFloat<$T: ty>) => $(#[$cfg:meta])* impl (OrderedFloat<$U: ty>) ) => {
+        $(#[$cfg])*
+        impl AsPrimitive<OrderedFloat<$U>> for OrderedFloat<$T> {
+            #[inline] fn as_(self) -> OrderedFloat<$U> { OrderedFloat(self.0 as $U) }
+        }
+    };
+    (@ ($T: ty) => $(#[$cfg:meta])* impl (OrderedFloat<$U: ty>) ) => {
+        $(#[$cfg])*
+        impl AsPrimitive<OrderedFloat<$U>> for $T {
+            #[inline] fn as_(self) -> OrderedFloat<$U> { OrderedFloat(self as $U) }
+        }
+    };
+    (@ (OrderedFloat<$T: ty>) => $(#[$cfg:meta])* impl ($U: ty) ) => {
+        $(#[$cfg])*
+        impl AsPrimitive<$U> for OrderedFloat<$T> {
+            #[inline] fn as_(self) -> $U { self.0 as $U }
+        }
+    };
+    ($T: tt => { $( $U: tt ),* } ) => {$(
+        impl_as_primitive!(@ $T => impl $U);
+    )*};
+}
+
+impl_as_primitive!((OrderedFloat<f32>) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((OrderedFloat<f64>) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+
+impl_as_primitive!((NotNan<f32>) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((NotNan<f64>) => { (NotNan<f32>), (NotNan<f64>) });
+
+impl_as_primitive!((u8) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((i8) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((u16) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((i16) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((u32) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((i32) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((u64) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((i64) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((usize) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((isize) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((f32) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+impl_as_primitive!((f64) => { (OrderedFloat<f32>), (OrderedFloat<f64>) });
+
+impl_as_primitive!((u8) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((i8) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((u16) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((i16) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((u32) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((i32) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((u64) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((i64) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((usize) => { (NotNan<f32>), (NotNan<f64>) });
+impl_as_primitive!((isize) => { (NotNan<f32>), (NotNan<f64>) });
+
+impl_as_primitive!((OrderedFloat<f32>) => { (u8), (u16), (u32), (u64), (usize), (i8), (i16), (i32), (i64), (isize), (f32), (f64) });
+impl_as_primitive!((OrderedFloat<f64>) => { (u8), (u16), (u32), (u64), (usize), (i8), (i16), (i32), (i64), (isize), (f32), (f64) });
+
+impl_as_primitive!((NotNan<f32>) => { (u8), (u16), (u32), (u64), (usize), (i8), (i16), (i32), (i64), (isize), (f32), (f64) });
+impl_as_primitive!((NotNan<f64>) => { (u8), (u16), (u32), (u64), (usize), (i8), (i16), (i32), (i64), (isize), (f32), (f64) });
+
 impl<T: FromPrimitive> FromPrimitive for OrderedFloat<T> {
     fn from_i64(n: i64) -> Option<Self> {
         T::from_i64(n).map(OrderedFloat)
@@ -932,14 +1005,13 @@ impl Borrow<f64> for NotNan<f64> {
 #[allow(clippy::derive_ord_xor_partial_ord)]
 impl<T: Float> Ord for NotNan<T> {
     fn cmp(&self, other: &NotNan<T>) -> Ordering {
-        match self.partial_cmp(&other) {
+        match self.partial_cmp(other) {
             Some(ord) => ord,
             None => unsafe { unreachable_unchecked() },
         }
     }
 }
 
-#[allow(clippy::derive_hash_xor_eq)]
 impl<T: Float> Hash for NotNan<T> {
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
@@ -954,6 +1026,18 @@ impl<T: Float + fmt::Display> fmt::Display for NotNan<T> {
     }
 }
 
+impl NotNan<f64> {
+    /// Converts this [`NotNan`]`<`[`f64`]`>` to a [`NotNan`]`<`[`f32`]`>` while giving up on
+    /// precision, [using `roundTiesToEven` as rounding mode, yielding `Infinity` on
+    /// overflow](https://doc.rust-lang.org/reference/expressions/operator-expr.html#semantics).
+    pub fn as_f32(self) -> NotNan<f32> {
+        // This is not destroying invariants, as it is a pure rounding operation. The only two special
+        // cases are where f32 would be overflowing, then the operation yields Infinity, or where
+        // the input is already NaN, in which case the invariant is already broken elsewhere.
+        NotNan(self.0 as f32)
+    }
+}
+
 impl From<NotNan<f32>> for f32 {
     #[inline]
     fn from(value: NotNan<f32>) -> Self {
@@ -1283,7 +1367,7 @@ fn raw_double_bits<F: Float>(f: &F) -> u64 {
     }
 
     let exp_u64 = exp as u16 as u64;
-    let sign_u64 = if sign > 0 { 1u64 } else { 0u64 };
+    let sign_u64 = (sign > 0) as u64;
     (man & MAN_MASK) | ((exp_u64 << 52) & EXP_MASK) | ((sign_u64 << 63) & SIGN_MASK)
 }
 
@@ -1490,6 +1574,42 @@ impl<T: Float> NumCast for NotNan<T> {
     }
 }
 
+macro_rules! impl_float_const_method {
+    ($wrapper:expr, $method:ident) => {
+        #[allow(non_snake_case)]
+        fn $method() -> Self {
+            $wrapper(T::$method())
+        }
+    };
+}
+
+macro_rules! impl_float_const {
+    ($type:ident, $wrapper:expr) => {
+        impl<T: FloatConst> FloatConst for $type<T> {
+            impl_float_const_method!($wrapper, E);
+            impl_float_const_method!($wrapper, FRAC_1_PI);
+            impl_float_const_method!($wrapper, FRAC_1_SQRT_2);
+            impl_float_const_method!($wrapper, FRAC_2_PI);
+            impl_float_const_method!($wrapper, FRAC_2_SQRT_PI);
+            impl_float_const_method!($wrapper, FRAC_PI_2);
+            impl_float_const_method!($wrapper, FRAC_PI_3);
+            impl_float_const_method!($wrapper, FRAC_PI_4);
+            impl_float_const_method!($wrapper, FRAC_PI_6);
+            impl_float_const_method!($wrapper, FRAC_PI_8);
+            impl_float_const_method!($wrapper, LN_10);
+            impl_float_const_method!($wrapper, LN_2);
+            impl_float_const_method!($wrapper, LOG10_E);
+            impl_float_const_method!($wrapper, LOG2_E);
+            impl_float_const_method!($wrapper, PI);
+            impl_float_const_method!($wrapper, SQRT_2);
+        }
+    };
+}
+
+impl_float_const!(OrderedFloat, OrderedFloat);
+// Float constants are not NaN.
+impl_float_const!(NotNan, |x| unsafe { NotNan::new_unchecked(x) });
+
 #[cfg(feature = "serde")]
 mod impl_serde {
     extern crate serde;
@@ -1655,6 +1775,93 @@ mod impl_rkyv {
     }
 }
 
+#[cfg(feature = "speedy")]
+mod impl_speedy {
+    use super::{NotNan, OrderedFloat};
+    use num_traits::Float;
+    use speedy::{Context, Readable, Reader, Writable, Writer};
+
+    impl<C, T> Writable<C> for OrderedFloat<T>
+    where
+        C: Context,
+        T: Writable<C>,
+    {
+        fn write_to<W: ?Sized + Writer<C>>(&self, writer: &mut W) -> Result<(), C::Error> {
+            self.0.write_to(writer)
+        }
+
+        fn bytes_needed(&self) -> Result<usize, C::Error> {
+            self.0.bytes_needed()
+        }
+    }
+
+    impl<C, T> Writable<C> for NotNan<T>
+    where
+        C: Context,
+        T: Writable<C>,
+    {
+        fn write_to<W: ?Sized + Writer<C>>(&self, writer: &mut W) -> Result<(), C::Error> {
+            self.0.write_to(writer)
+        }
+
+        fn bytes_needed(&self) -> Result<usize, C::Error> {
+            self.0.bytes_needed()
+        }
+    }
+
+    impl<'a, T, C: Context> Readable<'a, C> for OrderedFloat<T>
+    where
+        T: Readable<'a, C>,
+    {
+        fn read_from<R: Reader<'a, C>>(reader: &mut R) -> Result<Self, C::Error> {
+            T::read_from(reader).map(OrderedFloat)
+        }
+
+        fn minimum_bytes_needed() -> usize {
+            T::minimum_bytes_needed()
+        }
+    }
+
+    impl<'a, T: Float, C: Context> Readable<'a, C> for NotNan<T>
+    where
+        T: Readable<'a, C>,
+    {
+        fn read_from<R: Reader<'a, C>>(reader: &mut R) -> Result<Self, C::Error> {
+            let value: T = reader.read_value()?;
+            Self::new(value).map_err(|error| {
+                speedy::Error::custom(std::format!("failed to read NotNan: {}", error)).into()
+            })
+        }
+
+        fn minimum_bytes_needed() -> usize {
+            T::minimum_bytes_needed()
+        }
+    }
+
+    #[test]
+    fn test_ordered_float() {
+        let float = OrderedFloat(1.0f64);
+        let buffer = float.write_to_vec().unwrap();
+        let deser_float: OrderedFloat<f64> = OrderedFloat::read_from_buffer(&buffer).unwrap();
+        assert_eq!(deser_float, float);
+    }
+
+    #[test]
+    fn test_not_nan() {
+        let float = NotNan(1.0f64);
+        let buffer = float.write_to_vec().unwrap();
+        let deser_float: NotNan<f64> = NotNan::read_from_buffer(&buffer).unwrap();
+        assert_eq!(deser_float, float);
+    }
+
+    #[test]
+    fn test_not_nan_with_nan() {
+        let nan_buf = f64::nan().write_to_vec().unwrap();
+        let nan_err: Result<NotNan<f64>, _> = NotNan::read_from_buffer(&nan_buf);
+        assert!(nan_err.is_err());
+    }
+}
+
 #[cfg(all(feature = "std", feature = "schemars"))]
 mod impl_schemars {
     extern crate schemars;
@@ -1977,3 +2184,105 @@ mod impl_proptest {
     }
     impl_arbitrary! { f32, f64 }
 }
+
+#[cfg(feature = "arbitrary")]
+mod impl_arbitrary {
+    use super::{FloatIsNan, NotNan, OrderedFloat};
+    use arbitrary::{Arbitrary, Unstructured};
+    use num_traits::FromPrimitive;
+
+    macro_rules! impl_arbitrary {
+        ($($f:ident),+) => {
+            $(
+                impl<'a> Arbitrary<'a> for NotNan<$f> {
+                    fn arbitrary(u: &mut Unstructured<'a>) -> arbitrary::Result<Self> {
+                        let float: $f = u.arbitrary()?;
+                        match NotNan::new(float) {
+                            Ok(notnan_value) => Ok(notnan_value),
+                            Err(FloatIsNan) => {
+                                // If our arbitrary float input was a NaN (encoded by exponent = max
+                                // value), then replace it with a finite float, reusing the mantissa
+                                // bits.
+                                //
+                                // This means the output is not uniformly distributed among all
+                                // possible float values, but Arbitrary makes no promise that that
+                                // is true.
+                                //
+                                // An alternative implementation would be to return an
+                                // `arbitrary::Error`, but that is not as useful since it forces the
+                                // caller to retry with new random/fuzzed data; and the precendent of
+                                // `arbitrary`'s built-in implementations is to prefer the approach of
+                                // mangling the input bits to fit.
+
+                                let (mantissa, _exponent, sign) =
+                                    num_traits::Float::integer_decode(float);
+                                let revised_float = <$f>::from_i64(
+                                    i64::from(sign) * mantissa as i64
+                                ).unwrap();
+
+                                // If this unwrap() fails, then there is a bug in the above code.
+                                Ok(NotNan::new(revised_float).unwrap())
+                            }
+                        }
+                    }
+
+                    fn size_hint(depth: usize) -> (usize, Option<usize>) {
+                        <$f as Arbitrary>::size_hint(depth)
+                    }
+                }
+
+                impl<'a> Arbitrary<'a> for OrderedFloat<$f> {
+                    fn arbitrary(u: &mut Unstructured<'a>) -> arbitrary::Result<Self> {
+                        let float: $f = u.arbitrary()?;
+                        Ok(OrderedFloat::from(float))
+                    }
+
+                    fn size_hint(depth: usize) -> (usize, Option<usize>) {
+                        <$f as Arbitrary>::size_hint(depth)
+                    }
+                }
+            )*
+        }
+    }
+    impl_arbitrary! { f32, f64 }
+}
+
+#[cfg(feature = "bytemuck")]
+mod impl_bytemuck {
+    use super::{Float, NotNan, OrderedFloat};
+    use bytemuck::{AnyBitPattern, CheckedBitPattern, NoUninit, Pod, Zeroable};
+
+    unsafe impl<T: Zeroable> Zeroable for OrderedFloat<T> {}
+
+    // The zero bit pattern is indeed not a NaN bit pattern.
+    unsafe impl<T: Zeroable> Zeroable for NotNan<T> {}
+
+    unsafe impl<T: Pod> Pod for OrderedFloat<T> {}
+
+    // `NotNan<T>` can only implement `NoUninit` and not `Pod`, since not every bit pattern is
+    // valid (NaN bit patterns are invalid). `NoUninit` guarantees that we can read any bit pattern
+    // from the value, which is fine in this case.
+    unsafe impl<T: NoUninit> NoUninit for NotNan<T> {}
+
+    unsafe impl<T: Float + AnyBitPattern> CheckedBitPattern for NotNan<T> {
+        type Bits = T;
+
+        fn is_valid_bit_pattern(bits: &Self::Bits) -> bool {
+            !bits.is_nan()
+        }
+    }
+
+    #[test]
+    fn test_not_nan_bit_pattern() {
+        use bytemuck::checked::{try_cast, CheckedCastError};
+
+        let nan = f64::NAN;
+        assert_eq!(
+            try_cast::<f64, NotNan<f64>>(nan),
+            Err(CheckedCastError::InvalidBitPattern),
+        );
+
+        let pi = core::f64::consts::PI;
+        assert!(try_cast::<f64, NotNan<f64>>(pi).is_ok());
+    }
+}
