@@ -1,12 +1,14 @@
+#![allow(clippy::float_cmp, clippy::eq_op, clippy::op_ref)]
+
 extern crate num_traits;
 extern crate ordered_float;
 
-pub use ordered_float::*;
-pub use num_traits::{Bounded, FromPrimitive, Num, One, Signed, ToPrimitive, Zero};
-#[cfg(feature = "std")]
-pub use num_traits::Float;
 #[cfg(not(feature = "std"))]
 pub use num_traits::float::FloatCore as Float;
+#[cfg(feature = "std")]
+pub use num_traits::Float;
+pub use num_traits::{Bounded, FloatConst, FromPrimitive, Num, One, Signed, ToPrimitive, Zero};
+pub use ordered_float::*;
 
 pub use std::cmp::Ordering::*;
 pub use std::convert::TryFrom;
@@ -41,9 +43,18 @@ fn ordered_f32_compare_regular_floats_op() {
 #[test]
 fn ordered_f32_compare_nan() {
     let f32_nan: f32 = Float::nan();
-    assert_eq!(OrderedFloat(f32_nan).cmp(&OrderedFloat(Float::nan())), Equal);
-    assert_eq!(OrderedFloat(f32_nan).cmp(&OrderedFloat(-100000.0f32)), Greater);
-    assert_eq!(OrderedFloat(-100.0f32).cmp(&OrderedFloat(Float::nan())), Less);
+    assert_eq!(
+        OrderedFloat(f32_nan).cmp(&OrderedFloat(Float::nan())),
+        Equal
+    );
+    assert_eq!(
+        OrderedFloat(f32_nan).cmp(&OrderedFloat(-100000.0f32)),
+        Greater
+    );
+    assert_eq!(
+        OrderedFloat(-100.0f32).cmp(&OrderedFloat(Float::nan())),
+        Less
+    );
 }
 
 #[test]
@@ -142,7 +153,10 @@ fn not_nan32_signed() {
 
 #[test]
 fn not_nan32_num_cast() {
-    assert_eq!(<NotNan<f32> as num_traits::NumCast>::from(42).unwrap(), 42f32);
+    assert_eq!(
+        <NotNan<f32> as num_traits::NumCast>::from(42).unwrap(),
+        42f32
+    );
     assert_eq!(<NotNan<f32> as num_traits::NumCast>::from(f32::nan()), None);
 }
 
@@ -205,30 +219,15 @@ fn not_nan32_fail_when_constructing_with_nan() {
 
 #[test]
 fn not_nan32_calculate_correctly() {
-    assert_eq!(
-        *(not_nan(5.0f32) + not_nan(4.0f32)),
-        5.0f32 + 4.0f32
-    );
+    assert_eq!(*(not_nan(5.0f32) + not_nan(4.0f32)), 5.0f32 + 4.0f32);
     assert_eq!(*(not_nan(5.0f32) + 4.0f32), 5.0f32 + 4.0f32);
-    assert_eq!(
-        *(not_nan(5.0f32) - not_nan(4.0f32)),
-        5.0f32 - 4.0f32
-    );
+    assert_eq!(*(not_nan(5.0f32) - not_nan(4.0f32)), 5.0f32 - 4.0f32);
     assert_eq!(*(not_nan(5.0f32) - 4.0f32), 5.0f32 - 4.0f32);
-    assert_eq!(
-        *(not_nan(5.0f32) * not_nan(4.0f32)),
-        5.0f32 * 4.0f32
-    );
+    assert_eq!(*(not_nan(5.0f32) * not_nan(4.0f32)), 5.0f32 * 4.0f32);
     assert_eq!(*(not_nan(5.0f32) * 4.0f32), 5.0f32 * 4.0f32);
-    assert_eq!(
-        *(not_nan(8.0f32) / not_nan(4.0f32)),
-        8.0f32 / 4.0f32
-    );
+    assert_eq!(*(not_nan(8.0f32) / not_nan(4.0f32)), 8.0f32 / 4.0f32);
     assert_eq!(*(not_nan(8.0f32) / 4.0f32), 8.0f32 / 4.0f32);
-    assert_eq!(
-        *(not_nan(8.0f32) % not_nan(4.0f32)),
-        8.0f32 % 4.0f32
-    );
+    assert_eq!(*(not_nan(8.0f32) % not_nan(4.0f32)), 8.0f32 % 4.0f32);
     assert_eq!(*(not_nan(8.0f32) % 4.0f32), 8.0f32 % 4.0f32);
     assert_eq!(*(-not_nan(1.0f32)), -1.0f32);
 
@@ -262,36 +261,31 @@ fn not_nan32_calculate_correctly() {
     number %= 4.0f32;
     assert_eq!(*number, 1.0f32);
 
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f32);
-            tmp += f32::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f32);
-            tmp -= f32::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f32);
-            tmp *= f32::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f32);
-            tmp /= f32::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f32);
-            tmp %= f32::NAN;
-        }).is_err()
-    );
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f32);
+        tmp += f32::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f32);
+        tmp -= f32::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f32);
+        tmp *= f32::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f32);
+        tmp /= f32::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f32);
+        tmp %= f32::NAN;
+    })
+    .is_err());
 }
 
 #[test]
@@ -309,30 +303,15 @@ fn not_nan64_fail_when_constructing_with_nan() {
 
 #[test]
 fn not_nan64_calculate_correctly() {
-    assert_eq!(
-        *(not_nan(5.0f64) + not_nan(4.0f64)),
-        5.0f64 + 4.0f64
-    );
+    assert_eq!(*(not_nan(5.0f64) + not_nan(4.0f64)), 5.0f64 + 4.0f64);
     assert_eq!(*(not_nan(5.0f64) + 4.0f64), 5.0f64 + 4.0f64);
-    assert_eq!(
-        *(not_nan(5.0f64) - not_nan(4.0f64)),
-        5.0f64 - 4.0f64
-    );
+    assert_eq!(*(not_nan(5.0f64) - not_nan(4.0f64)), 5.0f64 - 4.0f64);
     assert_eq!(*(not_nan(5.0f64) - 4.0f64), 5.0f64 - 4.0f64);
-    assert_eq!(
-        *(not_nan(5.0f64) * not_nan(4.0f64)),
-        5.0f64 * 4.0f64
-    );
+    assert_eq!(*(not_nan(5.0f64) * not_nan(4.0f64)), 5.0f64 * 4.0f64);
     assert_eq!(*(not_nan(5.0f64) * 4.0f64), 5.0f64 * 4.0f64);
-    assert_eq!(
-        *(not_nan(8.0f64) / not_nan(4.0f64)),
-        8.0f64 / 4.0f64
-    );
+    assert_eq!(*(not_nan(8.0f64) / not_nan(4.0f64)), 8.0f64 / 4.0f64);
     assert_eq!(*(not_nan(8.0f64) / 4.0f64), 8.0f64 / 4.0f64);
-    assert_eq!(
-        *(not_nan(8.0f64) % not_nan(4.0f64)),
-        8.0f64 % 4.0f64
-    );
+    assert_eq!(*(not_nan(8.0f64) % not_nan(4.0f64)), 8.0f64 % 4.0f64);
     assert_eq!(*(not_nan(8.0f64) % 4.0f64), 8.0f64 % 4.0f64);
     assert_eq!(*(-not_nan(1.0f64)), -1.0f64);
 
@@ -366,36 +345,31 @@ fn not_nan64_calculate_correctly() {
     number %= 4.0f64;
     assert_eq!(*number, 1.0f64);
 
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f64);
-            tmp += f64::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f64);
-            tmp -= f64::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f64);
-            tmp *= f64::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f64);
-            tmp /= f64::NAN;
-        }).is_err()
-    );
-    assert!(
-        panic::catch_unwind(|| {
-            let mut tmp = not_nan(0.0f64);
-            tmp %= f64::NAN;
-        }).is_err()
-    );
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f64);
+        tmp += f64::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f64);
+        tmp -= f64::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f64);
+        tmp *= f64::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f64);
+        tmp /= f64::NAN;
+    })
+    .is_err());
+    assert!(panic::catch_unwind(|| {
+        let mut tmp = not_nan(0.0f64);
+        tmp %= f64::NAN;
+    })
+    .is_err());
 }
 
 #[test]
@@ -456,7 +430,10 @@ fn not_nan64_to_primitive() {
 
 #[test]
 fn not_nan64_num() {
-    assert_eq!(NotNan::<f64>::from_str_radix("42.0", 10).unwrap(), not_nan(42.0f64));
+    assert_eq!(
+        NotNan::<f64>::from_str_radix("42.0", 10).unwrap(),
+        not_nan(42.0f64)
+    );
     assert!(NotNan::<f64>::from_str_radix("NaN", 10).is_err());
 }
 
@@ -471,7 +448,10 @@ fn not_nan64_signed() {
 
 #[test]
 fn not_nan64_num_cast() {
-    assert_eq!(<NotNan<f64> as num_traits::NumCast>::from(42), Some(not_nan(42f64)));
+    assert_eq!(
+        <NotNan<f64> as num_traits::NumCast>::from(42),
+        Some(not_nan(42f64))
+    );
     assert_eq!(<NotNan<f64> as num_traits::NumCast>::from(f64::nan()), None);
 }
 
@@ -619,7 +599,6 @@ fn add() {
     assert_eq!(OrderedFloat(0.0) + OrderedFloat(0.0), 0.0);
     assert_eq!(OrderedFloat(0.0) + &OrderedFloat(0.0), 0.0);
     assert_eq!(&OrderedFloat(0.0) + OrderedFloat(0.0), 0.0);
-    assert_eq!(&OrderedFloat(0.0) + &OrderedFloat(0.0), 0.0);
     assert_eq!(OrderedFloat(0.0) + 0.0, 0.0);
     assert_eq!(OrderedFloat(0.0) + &0.0, 0.0);
     assert_eq!(&OrderedFloat(0.0) + 0.0, 0.0);
@@ -641,7 +620,7 @@ fn ordered_f64_neg() {
 fn test_sum_fails_on_nan() {
     let a = not_nan(std::f32::INFINITY);
     let b = not_nan(std::f32::NEG_INFINITY);
-    let _c: NotNan<_> = [a,b].iter().sum();
+    let _c: NotNan<_> = [a, b].iter().sum();
 }
 
 #[test]
@@ -649,7 +628,7 @@ fn test_sum_fails_on_nan() {
 fn test_product_fails_on_nan() {
     let a = not_nan(std::f32::INFINITY);
     let b = not_nan(0f32);
-    let _c: NotNan<_> = [a,b].iter().product();
+    let _c: NotNan<_> = [a, b].iter().product();
 }
 
 #[test]
@@ -658,21 +637,26 @@ fn not_nan64_sum_product() {
     let b = not_nan(132f64);
     let c = not_nan(5.1);
 
-    assert_eq!(std::iter::empty::<NotNan<f64>>().sum::<NotNan<_>>(), NotNan::new(0f64).unwrap());
+    assert_eq!(
+        std::iter::empty::<NotNan<f64>>().sum::<NotNan<_>>(),
+        NotNan::new(0f64).unwrap()
+    );
     assert_eq!([a].iter().sum::<NotNan<_>>(), a);
-    assert_eq!([a,b].iter().sum::<NotNan<_>>(), a + b);
-    assert_eq!([a,b,c].iter().sum::<NotNan<_>>(), a + b + c);
+    assert_eq!([a, b].iter().sum::<NotNan<_>>(), a + b);
+    assert_eq!([a, b, c].iter().sum::<NotNan<_>>(), a + b + c);
 
-    assert_eq!(std::iter::empty::<NotNan<f64>>().product::<NotNan<_>>(), NotNan::new(1f64).unwrap());
+    assert_eq!(
+        std::iter::empty::<NotNan<f64>>().product::<NotNan<_>>(),
+        NotNan::new(1f64).unwrap()
+    );
     assert_eq!([a].iter().product::<NotNan<_>>(), a);
-    assert_eq!([a,b].iter().product::<NotNan<_>>(), a * b);
-    assert_eq!([a,b,c].iter().product::<NotNan<_>>(), a * b * c);
-
+    assert_eq!([a, b].iter().product::<NotNan<_>>(), a * b);
+    assert_eq!([a, b, c].iter().product::<NotNan<_>>(), a * b * c);
 }
 
 #[test]
 fn not_nan_usage_in_const_context() {
-    const A: NotNan<f32> = unsafe { NotNan::unchecked_new(111f32) };
+    const A: NotNan<f32> = unsafe { NotNan::new_unchecked(111f32) };
     assert_eq!(A, NotNan::new(111f32).unwrap());
 }
 
@@ -680,7 +664,8 @@ fn not_nan_usage_in_const_context() {
 fn not_nan_panic_safety() {
     let catch_op = |mut num, op: fn(&mut NotNan<_>)| {
         let mut num_ref = panic::AssertUnwindSafe(&mut num);
-        let _ = panic::catch_unwind(move || op(&mut *num_ref));
+        #[allow(clippy::needless_borrow)] // mut-borrow needed for msrv 1.36.0
+        let _ = panic::catch_unwind(move || op(&mut num_ref));
         num
     };
 
@@ -707,3 +692,81 @@ fn from_ref() {
     assert_eq!(*o, 2.0f64);
     assert_eq!(f, 2.0f64);
 }
+
+macro_rules! test_float_const_method {
+    ($type:ident < $inner:ident >, $method:ident) => {
+        assert_eq!($type::<$inner>::$method().into_inner(), $inner::$method())
+    };
+}
+
+macro_rules! test_float_const_methods {
+    ($type:ident < $inner:ident >) => {
+        test_float_const_method!($type<$inner>, E);
+        test_float_const_method!($type<$inner>, FRAC_1_PI);
+        test_float_const_method!($type<$inner>, FRAC_1_SQRT_2);
+        test_float_const_method!($type<$inner>, FRAC_2_PI);
+        test_float_const_method!($type<$inner>, FRAC_2_SQRT_PI);
+        test_float_const_method!($type<$inner>, FRAC_PI_2);
+        test_float_const_method!($type<$inner>, FRAC_PI_3);
+        test_float_const_method!($type<$inner>, FRAC_PI_4);
+        test_float_const_method!($type<$inner>, FRAC_PI_6);
+        test_float_const_method!($type<$inner>, FRAC_PI_8);
+        test_float_const_method!($type<$inner>, LN_10);
+        test_float_const_method!($type<$inner>, LN_2);
+        test_float_const_method!($type<$inner>, LOG10_E);
+        test_float_const_method!($type<$inner>, LOG2_E);
+        test_float_const_method!($type<$inner>, PI);
+        test_float_const_method!($type<$inner>, SQRT_2);
+    };
+}
+
+#[test]
+fn float_consts_equal_inner() {
+    test_float_const_methods!(OrderedFloat<f64>);
+    test_float_const_methods!(OrderedFloat<f32>);
+    test_float_const_methods!(NotNan<f64>);
+    test_float_const_methods!(NotNan<f32>);
+}
+
+#[cfg(feature = "arbitrary")]
+mod arbitrary_test {
+    use super::{NotNan, OrderedFloat};
+    use arbitrary::{Arbitrary, Unstructured};
+
+    #[test]
+    fn exhaustive() {
+        // Exhaustively search all patterns of sign and exponent bits plus a few mantissa bits.
+        for high_bytes in 0..=u16::MAX {
+            let [h1, h2] = high_bytes.to_be_bytes();
+
+            // Each of these should not
+            //   * panic,
+            //   * return an error, or
+            //   * need more bytes than given.
+            let n32: NotNan<f32> = Unstructured::new(&[h1, h2, h1, h2])
+                .arbitrary()
+                .expect("NotNan<f32> failure");
+            let n64: NotNan<f64> = Unstructured::new(&[h1, h2, h1, h2, h1, h2, h1, h2])
+                .arbitrary()
+                .expect("NotNan<f64> failure");
+            let _: OrderedFloat<f32> = Unstructured::new(&[h1, h2, h1, h2])
+                .arbitrary()
+                .expect("OrderedFloat<f32> failure");
+            let _: OrderedFloat<f64> = Unstructured::new(&[h1, h2, h1, h2, h1, h2, h1, h2])
+                .arbitrary()
+                .expect("OrderedFloat<f64> failure");
+
+            // Check for violation of NotNan's property of never containing a NaN.
+            assert!(!n32.into_inner().is_nan());
+            assert!(!n64.into_inner().is_nan());
+        }
+    }
+
+    #[test]
+    fn size_hints() {
+        assert_eq!(NotNan::<f32>::size_hint(0), (4, Some(4)));
+        assert_eq!(NotNan::<f64>::size_hint(0), (8, Some(8)));
+        assert_eq!(OrderedFloat::<f32>::size_hint(0), (4, Some(4)));
+        assert_eq!(OrderedFloat::<f64>::size_hint(0), (8, Some(8)));
+    }
+}
